@@ -21,13 +21,14 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import units
 
+import glance.async_
 from glance.common import exception
-from glance.common import wsgi
 from glance.i18n import _, _LE, _LW
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
+GLANCE_RESERVED_NS = 'os_glance'
 _CACHED_THREAD_POOL = {}
 
 
@@ -112,13 +113,13 @@ def get_remaining_quota(context, db_api, image_id=None):
 
     # set quota must have a number optionally followed by B, KB, MB,
     # GB or TB without any spaces in between
-    pattern = re.compile('^(\d+)((K|M|G|T)?B)?$')
+    pattern = re.compile(r'^(\d+)((K|M|G|T)?B)?$')
     match = pattern.match(users_quota)
 
     if not match:
         LOG.error(_LE("Invalid value for option user_storage_quota: "
-                      "%(users_quota)s")
-                  % {'users_quota': users_quota})
+                      "%(users_quota)s"),
+                  {'users_quota': users_quota})
         raise exception.InvalidOptionValue(option='user_storage_quota',
                                            value=users_quota)
 
@@ -166,8 +167,8 @@ def check_quota(context, image_size, db_api, image_id=None):
         if remaining <= 0:
             LOG.warn(_LW("User %(user)s attempted to upload an image of"
                          " unknown size that will exceed the quota."
-                         " %(remaining)d bytes remaining.")
-                     % {'user': user, 'remaining': remaining})
+                         " %(remaining)d bytes remaining."),
+                     {'user': user, 'remaining': remaining})
             raise exception.StorageQuotaFull(image_size=image_size,
                                              remaining=remaining)
         return
@@ -175,8 +176,8 @@ def check_quota(context, image_size, db_api, image_id=None):
     if image_size > remaining:
         LOG.warn(_LW("User %(user)s attempted to upload an image of size"
                      " %(size)d that will exceed the quota. %(remaining)d"
-                     " bytes remaining.")
-                 % {'user': user, 'size': image_size, 'remaining': remaining})
+                     " bytes remaining."),
+                 {'user': user, 'size': image_size, 'remaining': remaining})
         raise exception.StorageQuotaFull(image_size=image_size,
                                          remaining=remaining)
 
@@ -197,20 +198,33 @@ def memoize(lock_name):
     return memoizer_wrapper
 
 
-def get_thread_pool(lock_name, size=1024):
-    """Initializes eventlet thread pool.
+# NOTE(danms): This is the default pool size that will be used for
+# the get_thread_pool() cache wrapper below. This is a global here
+# because it needs to be overridden for the pure-wsgi mode in
+# wsgi_app.py where native threads are used.
+DEFAULT_POOL_SIZE = 1024
+
+
+def get_thread_pool(lock_name, size=None):
+    """Initializes thread pool.
 
     If thread pool is present in cache, then returns it from cache
     else create new pool, stores it in cache and return newly created
     pool.
 
     @param lock_name:  Name of the lock.
-    @param size: Size of eventlet pool.
+    @param size: Size of pool.
 
-    @return: eventlet pool
+    @return: ThreadPoolModel
     """
+
+    if size is None:
+        size = DEFAULT_POOL_SIZE
+
     @memoize(lock_name)
     def _get_thread_pool():
-        return wsgi.get_asynchronous_eventlet_pool(size=size)
+        threadpool_cls = glance.async_.get_threadpool_model()
+        LOG.debug('Initializing named threadpool %r', lock_name)
+        return threadpool_cls(size)
 
     return _get_thread_pool
