@@ -21,8 +21,6 @@
 Glance Management Utility
 """
 
-from __future__ import print_function
-
 # FIXME(sirp): When we have glance-admin we can consider merging this into it
 # Perhaps for consistency with Nova, we would then rename glance-admin ->
 # glance-manage (or the other way around)
@@ -547,9 +545,7 @@ def main():
         logging.register_options(CONF)
         CONF.set_default(name='use_stderr', default=True)
         cfg_files = cfg.find_config_files(project='glance',
-                                          prog='glance-registry')
-        cfg_files.extend(cfg.find_config_files(project='glance',
-                                               prog='glance-api'))
+                                          prog='glance-api')
         cfg_files.extend(cfg.find_config_files(project='glance',
                                                prog='glance-manage'))
         config.parse_args(default_config_files=cfg_files)
