@@ -16,6 +16,7 @@
 import sys
 from unittest import mock
 
+from glance_store import exceptions as store_exceptions
 from oslo_config import cfg
 from oslo_utils import units
 import taskflow
@@ -26,7 +27,6 @@ from glance.common.scripts.image_import import main as image_import
 from glance import context
 from glance import gateway
 import glance.tests.utils as test_utils
-from glance_store import exceptions as store_exceptions
 
 from cursive import exception as cursive_exception
 
@@ -63,8 +63,11 @@ class TestApiImageImportTask(test_utils.BaseTestCase):
 
         self.mock_task_repo = mock.MagicMock()
         self.mock_image_repo = mock.MagicMock()
-        self.mock_image_repo.get.return_value.extra_properties = {
-            'os_glance_import_task': TASK_ID1}
+        self.mock_image = self.mock_image_repo.get.return_value
+        self.mock_image.extra_properties = {
+            'os_glance_import_task': TASK_ID1,
+            'os_glance_stage_host': 'http://glance2',
+        }
 
     @mock.patch('glance.async_.flows.api_image_import._VerifyStaging.__init__')
     @mock.patch('taskflow.patterns.linear_flow.Flow.add')
@@ -103,6 +106,17 @@ class TestApiImageImportTask(test_utils.BaseTestCase):
         self._pass_uri(uri=test_uri, file_uri=expected_uri,
                        import_req=self.gd_task_input['import_req'])
 
+    def test_get_flow_pops_stage_host(self):
+        import_flow.get_flow(task_id=TASK_ID1, task_type=TASK_TYPE,
+                             task_repo=self.mock_task_repo,
+                             image_repo=self.mock_image_repo,
+                             image_id=IMAGE_ID1,
+                             import_req=self.gd_task_input['import_req'])
+        self.assertNotIn('os_glance_stage_host',
+                         self.mock_image.extra_properties)
+        self.assertIn('os_glance_import_task',
+                      self.mock_image.extra_properties)
+
 
 class TestImageLock(test_utils.BaseTestCase):
     def setUp(self):
@@ -525,45 +539,6 @@ class TestDeleteFromFS(test_utils.BaseTestCase):
         mock_unlink.assert_not_called()
 
 
-class TestVerifyImageStateTask(test_utils.BaseTestCase):
-    def test_verify_active_status(self):
-        fake_img = mock.MagicMock(status='active',
-                                  extra_properties={
-                                      'os_glance_import_task': TASK_ID1})
-        mock_repo = mock.MagicMock()
-        mock_repo.get.return_value = fake_img
-        wrapper = import_flow.ImportActionWrapper(mock_repo, IMAGE_ID1,
-                                                  TASK_ID1)
-
-        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
-                                             wrapper, 'anything!')
-
-        task.execute()
-
-        fake_img.status = 'importing'
-        self.assertRaises(import_flow._NoStoresSucceeded,
-                          task.execute)
-
-    def test_revert_copy_status_unchanged(self):
-        wrapper = mock.MagicMock()
-        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
-                                             wrapper, 'copy-image')
-        task.revert(mock.sentinel.result)
-
-        # If we are doing copy-image, no state update should be made
-        wrapper.__enter__.return_value.set_image_status.assert_not_called()
-
-    def test_reverts_state_nocopy(self):
-        wrapper = mock.MagicMock()
-        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
-                                             wrapper, 'glance-direct')
-        task.revert(mock.sentinel.result)
-
-        # Except for copy-image, image state should revert to queued
-        action = wrapper.__enter__.return_value
-        action.set_image_status.assert_called_once_with('queued')
-
-
 class TestImportCopyImageTask(test_utils.BaseTestCase):
 
     def setUp(self):
@@ -603,6 +578,45 @@ class TestImportCopyImageTask(test_utils.BaseTestCase):
         img_repo.save.assert_not_called()
 
 
+class TestVerifyImageStateTask(test_utils.BaseTestCase):
+    def test_verify_active_status(self):
+        fake_img = mock.MagicMock(status='active',
+                                  extra_properties={
+                                      'os_glance_import_task': TASK_ID1})
+        mock_repo = mock.MagicMock()
+        mock_repo.get.return_value = fake_img
+        wrapper = import_flow.ImportActionWrapper(mock_repo, IMAGE_ID1,
+                                                  TASK_ID1)
+
+        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
+                                             wrapper, 'anything!')
+
+        task.execute()
+
+        fake_img.status = 'importing'
+        self.assertRaises(import_flow._NoStoresSucceeded,
+                          task.execute)
+
+    def test_revert_copy_status_unchanged(self):
+        wrapper = mock.MagicMock()
+        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
+                                             wrapper, 'copy-image')
+        task.revert(mock.sentinel.result)
+
+        # If we are doing copy-image, no state update should be made
+        wrapper.__enter__.return_value.set_image_attribute.assert_not_called()
+
+    def test_reverts_state_nocopy(self):
+        wrapper = mock.MagicMock()
+        task = import_flow._VerifyImageState(TASK_ID1, TASK_TYPE,
+                                             wrapper, 'glance-direct')
+        task.revert(mock.sentinel.result)
+
+        # Except for copy-image, image state should revert to queued
+        action = wrapper.__enter__.return_value
+        action.set_image_attribute.assert_called_once_with(status='queued')
+
+
 class TestImportActionWrapper(test_utils.BaseTestCase):
     def test_wrapper_success(self):
         mock_repo = mock.MagicMock()
@@ -649,7 +663,7 @@ class TestImportActionWrapper(test_utils.BaseTestCase):
 
         mock_image.status = 'foo'
         with wrapper as action:
-            action.set_image_status('bar')
+            action.set_image_attribute(status='bar')
 
         mock_log.debug.assert_called_once_with(
             'Image %(image_id)s status changing from '
@@ -665,6 +679,100 @@ class TestImportActionWrapper(test_utils.BaseTestCase):
                                                   TASK_ID1)
         self.assertEqual(IMAGE_ID1, wrapper.image_id)
 
+    def test_set_image_attribute(self):
+        mock_repo = mock.MagicMock()
+        mock_image = mock_repo.get.return_value
+        mock_image.extra_properties = {'os_glance_import_task': TASK_ID1}
+        mock_image.status = 'bar'
+        wrapper = import_flow.ImportActionWrapper(mock_repo, IMAGE_ID1,
+                                                  TASK_ID1)
+        with wrapper as action:
+            action.set_image_attribute(status='foo', virtual_size=123)
+        mock_repo.save.assert_called_once_with(mock_image, 'bar')
+        self.assertEqual('foo', mock_image.status)
+        self.assertEqual(123, mock_image.virtual_size)
+
+    def test_set_image_attribute_disallowed(self):
+        mock_repo = mock.MagicMock()
+        mock_image = mock_repo.get.return_value
+        mock_image.extra_properties = {'os_glance_import_task': TASK_ID1}
+        mock_image.status = 'bar'
+        wrapper = import_flow.ImportActionWrapper(mock_repo, IMAGE_ID1,
+                                                  TASK_ID1)
+        with wrapper as action:
+            self.assertRaises(AttributeError,
+                              action.set_image_attribute, id='foo')
+
+    @mock.patch.object(import_flow, 'LOG')
+    def test_set_image_extra_properties(self, mock_log):
+        mock_repo = mock.MagicMock()
+        mock_image = mock_repo.get.return_value
+        mock_image.image_id = IMAGE_ID1
+        mock_image.extra_properties = {'os_glance_import_task': TASK_ID1}
+        mock_image.status = 'bar'
+        wrapper = import_flow.ImportActionWrapper(mock_repo, IMAGE_ID1,
+                                                  TASK_ID1)
+        # One banned property
+        with wrapper as action:
+            action.set_image_extra_properties({'os_glance_foo': 'bar'})
+        self.assertEqual({'os_glance_import_task': TASK_ID1},
+                         mock_image.extra_properties)
+
+        mock_log.warning.assert_called()
+        mock_log.warning.reset_mock()
+
+        # Two banned properties
+        with wrapper as action:
+            action.set_image_extra_properties({'os_glance_foo': 'bar',
+                                               'os_glance_baz': 'bat'})
+        self.assertEqual({'os_glance_import_task': TASK_ID1},
+                         mock_image.extra_properties)
+
+        mock_log.warning.assert_called()
+        mock_log.warning.reset_mock()
+
+        # One banned and one allowed property
+        with wrapper as action:
+            action.set_image_extra_properties({'foo': 'bar',
+                                               'os_glance_foo': 'baz'})
+        self.assertEqual({'foo': 'bar',
+                          'os_glance_import_task': TASK_ID1},
+                         mock_image.extra_properties)
+
+        mock_log.warning.assert_called_once_with(
+            'Dropping %(key)s=%(val)s during metadata injection for %(image)s',
+            {'key': 'os_glance_foo', 'val': 'baz',
+             'image': IMAGE_ID1})
+
+    def test_image_size(self):
+        mock_repo = mock.MagicMock()
+        mock_image = mock_repo.get.return_value
+        mock_image.image_id = IMAGE_ID1
+        mock_image.extra_properties = {'os_glance_import_task': TASK_ID1}
+        mock_image.size = 123
+        wrapper = import_flow.ImportActionWrapper(mock_repo, IMAGE_ID1,
+                                                  TASK_ID1)
+        with wrapper as action:
+            self.assertEqual(123, action.image_size)
+
+    def test_image_locations(self):
+        mock_repo = mock.MagicMock()
+        mock_image = mock_repo.get.return_value
+        mock_image.image_id = IMAGE_ID1
+        mock_image.extra_properties = {'os_glance_import_task': TASK_ID1}
+        mock_image.locations = {'some': {'complex': ['structure']}}
+        wrapper = import_flow.ImportActionWrapper(mock_repo, IMAGE_ID1,
+                                                  TASK_ID1)
+        with wrapper as action:
+            self.assertEqual({'some': {'complex': ['structure']}},
+                             action.image_locations)
+            # Mutate our copy
+            action.image_locations['foo'] = 'bar'
+
+        # Make sure we did not mutate the image itself
+        self.assertEqual({'some': {'complex': ['structure']}},
+                         mock_image.locations)
+
     def test_drop_lock_for_task(self):
         mock_repo = mock.MagicMock()
         mock_repo.get.return_value.extra_properties = {
@@ -899,6 +1007,17 @@ class TestImportActions(test_utils.BaseTestCase):
                 _('Unexpected exception when deleting from store foo.'))
             mock_log.warning.reset_mock()
 
+    def test_pop_extra_property(self):
+        self.image.extra_properties = {'foo': '1', 'bar': 2}
+
+        # Should remove, if present
+        self.actions.pop_extra_property('foo')
+        self.assertEqual({'bar': 2}, self.image.extra_properties)
+
+        # Should not raise if missing
+        self.actions.pop_extra_property('baz')
+        self.assertEqual({'bar': 2}, self.image.extra_properties)
+
 
 @mock.patch('glance.common.scripts.utils.get_task')
 class TestCompleteTask(test_utils.BaseTestCase):
